/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;

/**
 * Representation of the structure style update output builder.
 */
public class StructureStyleNemoUpdateOutputBuilder extends DefaultOutputBuilderBase {

    private Map<Class<? extends StructureStyleNemoUpdateOutput>,
            StructureStyleNemoUpdateOutput>
            augmentation = new HashMap<>();

    /**
     * Constructs a structure style update output builder.
     */
    public StructureStyleNemoUpdateOutputBuilder() {
    }

    /**
     * Constructs a structure style update output builder
     * from a RPC result.
     *
     * @param rpc common RPC result
     */
    public StructureStyleNemoUpdateOutputBuilder(CommonRpcResult rpc) {
        resultCode = rpc.getResultCode();
        message = rpc.getMessage();
    }

    /**
     * Constructs structure style update output builder from
     * a structure style update output object.
     *
     * @param base structure style update output object
     */
    public StructureStyleNemoUpdateOutputBuilder(
            StructureStyleNemoUpdateOutput base) {
        message = base.getMessage();
        resultCode = base.getResultCode();
        if (base instanceof StructureStyleNemoUpdateOutputImpl) {
            StructureStyleNemoUpdateOutputImpl impl = (StructureStyleNemoUpdateOutputImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Adds augmentation by a class type and a structure style update output.
     * If the augmentation is null, remove the augmentation type key from
     * the hash map.
     *
     * @param augmentationType type that extends StructureStyleNemoUpdateOutput
     * @param augmentation StructureStyleNemoUpdateOutput object
     * @return self
     */
    public StructureStyleNemoUpdateOutputBuilder addAugmentation(
            Class<? extends StructureStyleNemoUpdateOutput> augmentationType,
            StructureStyleNemoUpdateOutput augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation by a class type.
     *
     * @param augmentationType type that extends StructureStyleNemoUpdateOutput
     * @return self
     */
    public StructureStyleNemoUpdateOutputBuilder removeAugmentation(
            Class<? extends StructureStyleNemoUpdateOutput> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds the structure style update output object.
     *
     * @return the structure style update output object
     */
    public StructureStyleNemoUpdateOutput build() {
        return new StructureStyleNemoUpdateOutputImpl(this);
    }

    /**
     * Representation of the implement of StructureStyleNemoUpdateOutput.
     */
    private static final class StructureStyleNemoUpdateOutputImpl
            extends DefaultOutputImplBase
            implements StructureStyleNemoUpdateOutput {

        private final Map<Class<? extends StructureStyleNemoUpdateOutput>,
                StructureStyleNemoUpdateOutput> augmentation;

        private StructureStyleNemoUpdateOutputImpl(
                StructureStyleNemoUpdateOutputBuilder base) {
            message = base.message;
            resultCode = base.resultCode;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public int hashCode() {
            return Objects.hash(message, resultCode, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            StructureStyleNemoUpdateOutputImpl other =
                    (StructureStyleNemoUpdateOutputImpl) obj;
            return Objects.equals(message, other.getMessage()) &&
                    Objects.equals(resultCode, other.getResultCode()) &&
                    Objects.equals(augmentation, other.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("message", message)
                    .add("resultCode", resultCode)
                    .add("augmentation", augmentation).toString();
        }
    }
}