/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.object.match.item.instance;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.onosproject.nemo.model.object.match.item.instance.match.item.value.RangeValue;

import com.google.common.collect.Maps;


public class MatchItemValueBuilder {

    Map<Class<? extends MatchItemValue>, MatchItemValue>
            augmentation = Maps.newHashMap();
    private Long intValue;
    private RangeValue rangeValue;
    private String stringValue;

    /**
     * Creates a default match item value builder.
     */
    public MatchItemValueBuilder() {
    }

    /**
     * Creates an match item value builder from a given match item value.
     *
     * @param base the action definition
     */
    public MatchItemValueBuilder(MatchItemValue base) {
        intValue = base.getIntValue();
        rangeValue = base.getRangeValue();
        stringValue = base.getStringValue();
        MatchItemValueImpl impl = (MatchItemValueImpl) base;
        if (!impl.augmentation.isEmpty()) {
            this.augmentation = new HashMap<>(impl.augmentation);
        }
    }

    /**
     * Sets the value to be used by the builder.
     *
     * @param value the value
     * @return self
     */
    public MatchItemValueBuilder IntValue(Long value) {
        intValue = value;
        return this;
    }

    /**
     * Sets the range value to be used by the builder.
     *
     * @param value the range value
     * @return self
     */
    public MatchItemValueBuilder RangeValue(RangeValue value) {
        rangeValue = value;
        return this;
    }

    /**
     * Sets the string value to be used by the builder.
     *
     * @param value the string value
     * @return self
     */
    public MatchItemValueBuilder StringValue(String value) {
        stringValue = value;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public MatchItemValueBuilder addAugmentation(
            Class<? extends MatchItemValue> augmentationType,
            MatchItemValue augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public MatchItemValueBuilder removeAugmentation(
            Class<? extends MatchItemValue> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds an match item value from this builder instance.
     *
     * @return match item value
     */
    public MatchItemValue build() {
        return new MatchItemValueImpl(this);
    }

    /**
     * The implementation of match item value.
     */
    private static final class MatchItemValueImpl implements MatchItemValue {

        private final Long intValue;
        private final RangeValue rangeValue;
        private final String stringValue;
        private final Map<Class<? extends MatchItemValue>, MatchItemValue>
                augmentation;

        private MatchItemValueImpl(MatchItemValueBuilder base) {
            intValue = base.intValue;
            rangeValue = base.rangeValue;
            stringValue = base.stringValue;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public Long getIntValue() {
            return intValue;
        }

        @Override
        public RangeValue getRangeValue() {
            return rangeValue;
        }

        @Override
        public String getStringValue() {
            return stringValue;
        }

        @Override
        public int hashCode() {
            return Objects.hash(
                    intValue, rangeValue, stringValue, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            MatchItemValue other = (MatchItemValue) obj;
            if (!Objects.equals(intValue, other.getIntValue())) {
                return false;
            }
            if (!Objects.equals(rangeValue, other.getRangeValue())) {
                return false;
            }
            if (!Objects.equals(stringValue, other.getStringValue())) {
                return false;
            }
            MatchItemValueImpl otherImpl = (MatchItemValueImpl) obj;
            return Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("intValue", intValue)
                    .add("rangeValue", rangeValue)
                    .add("stringValue", stringValue)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}