/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.object.node.instance;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.onosproject.nemo.model.common.PropertyName;
import org.onosproject.nemo.model.object.PropertyInstance;
import org.onosproject.nemo.model.object.property.instance.PropertyValues;

import com.google.common.collect.Maps;

/**
 * The builder of property.
 */
public class PropertyBuilder {

    private Map<Class<? extends Property>, Property> augmentation =
            Maps.newHashMap();
    private PropertyKey key;
    private PropertyName propertyName;
    private PropertyValues propertyValues;

    /**
     * Creates a default user role builder.
     */
    public PropertyBuilder() {
    }

    /**
     * Creates a property builder from a property.
     *
     * @param instance the property instance
     */
    public PropertyBuilder(PropertyInstance instance) {
        propertyName = instance.getPropertyName();
        propertyValues = instance.getPropertyValues();
    }

    /**
     * Creates a property builder from a property.
     *
     * @param base the property instance
     */
    public PropertyBuilder(Property base) {
        if (base.getKey() == null) {
            key = new PropertyKey(base.getPropertyName());
            propertyName = base.getPropertyName();
        } else {
            key = base.getKey();
            propertyName = key.getPropertyName();
        }
        propertyValues = base.getPropertyValues();
    }

    /**
     * Sets the key to be used by the builder.
     *
     * @param key key value
     * @return self
     */
    public PropertyBuilder key(PropertyKey key) {
        this.key = key;
        return this;
    }

    /**
     * Sets the property name to be used by the builder.
     *
     * @param name property name value
     * @return self
     */
    public PropertyBuilder propertyName(PropertyName name) {
        propertyName = name;
        return this;
    }

    /**
     * Sets the property value to be used by the builder.
     *
     * @param values property values
     * @return self
     */
    public PropertyBuilder propertyValues(PropertyValues values) {
        propertyValues = values;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public PropertyBuilder addAugmentation(Class<Property> augmentationType,
            Property augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public PropertyBuilder removeAugmentation(Class<Property>
            augmentationType) {
            augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a property from this builder instance.
     *
     * @return property
     */
    public Property build() {
        return new PropertyImpl(this);
    }

    /**
     * The implementation of property interface.
     */
    private static final class PropertyImpl implements Property {

        private final PropertyKey key;
        private final PropertyName propertyName;
        private final PropertyValues propertyValues;
        private final Map<Class<? extends Property>, Property> augmentation;

        private PropertyImpl(PropertyBuilder base) {
            if (base.key == null) {
                key = new PropertyKey(base.propertyName);
                propertyName = base.propertyName;
            } else {
                key = base.key;
                propertyName = key.getPropertyName();
            }
            this.propertyValues = base.propertyValues;
                augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public PropertyKey getKey() {
            return key;
        }

        @Override
        public PropertyName getPropertyName() {
            return propertyName;
        }

        @Override
        public PropertyValues getPropertyValues() {
            return propertyValues;
        }

        @Override
        public int hashCode() {
            return Objects.hash(key, propertyName, propertyValues,
                    augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            PropertyImpl o = (PropertyImpl) obj;
            return Objects.equals(key, o.key) &&
                    Objects.equals(propertyName, o.propertyName) &&
                    Objects.equals(propertyValues, o.propertyValues) &&
                    Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("key", key)
                    .add("propertyName", propertyName)
                    .add("propertyValues", propertyValues)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}