/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.object.property.definitions;

import static com.google.common.collect.ImmutableMap.Builder;
import static com.google.common.collect.ImmutableMap.builder;
import org.onosproject.nemo.model.common.PropertyName;

/**
 * Single property definition's information.
 */
public interface PropertyDefinition {

    /**
     * Returns a unique name for a kind of property.
     *
     * @return a name of property
     */
    PropertyName getPropertyName();

    /**
     * Returns the type of the property value.
     *
     * @return the value
     */
    PropertyValueType getPropertyValueType();

    /**
     * Specify whether the property is required for the object.
     *
     * @return required
     */
    IsRequired getIsRequired();

    /**
     * Specify whether the property is read-only for object.
     *
     * @return access
     */
    Access getIsReadOnly();

    /**
     * .
     * Returns Primary Key of YANG List Type
     *
     * @return the key
     */
    PropertyDefinitionKey getKey();

    /**
     * Designates property value type.
     */
    enum PropertyValueType {

        /** An string-valued property. */
        STRING(0),

        /** An integer-valued property. */
        INT(1),

        /** An integer-range property. */
        RANGE(2);

        int value;

        private static final java.util.Map<Integer, PropertyValueType>
                VALUE_MAP;

        PropertyValueType(int value) {
            this.value = value;
        }

        /**
         * Returns integer value.
         *
         * @return integer value
         */
        public int intValue() {
            return value;
        }

        static {
            Builder<Integer, PropertyValueType> b = builder();
            for (PropertyValueType enumItem : PropertyValueType.values()) {
                b.put(enumItem.value, enumItem);
            }
            VALUE_MAP = b.build();
        }

        /**
         * Returns property value type.
         *
         * @param value the value for value map
         * @return corresponding PropertyValueType item
         * @throws IllegalArgumentException if there is no mapping
         */
        public static PropertyValueType forValue(int value) {
            PropertyValueType is = VALUE_MAP.get(value);
            if (is == null) {
                throw new IllegalArgumentException(
                        "No PropertyValueType for value: " + value);
            }
            return is;
        }
    }

    /**
     * Designates required property.
     */
    enum IsRequired {

        /** The property is required */
        REQUIRED(0),

        /** The property is optional */
        OPTIONAL(1);

        int value;

        private static final java.util.Map<Integer, IsRequired> VALUE_MAP;


        IsRequired(int value) {
            this.value = value;
        }

        /**
         * Returns integer value.
         *
         * @return integer value
         */
        public int intValue() {
            return value;
        }

        static {
            Builder<Integer, IsRequired> b = builder();
            for (IsRequired enumItem : IsRequired.values()) {
                b.put(enumItem.value, enumItem);
            }
            VALUE_MAP = b.build();
        }

        /**
         * Returns value map.
         *
         * @param value is required value to get value map
         * @return corresponding IsRequired item
         * @throws IllegalArgumentException if there is no mapping
         */
        public static IsRequired forValue(int value) {
            IsRequired is = VALUE_MAP.get(value);
            if(is == null){
                throw new IllegalArgumentException(
                        "No PropertyValueType for value: " + value);
            }
            return is;
        }
    }

    /**
     * Designates property access.
     */
    enum Access {

        /** The property can be read and written. */
        READ_WRITE(0),

        /** The property is read-only. */
        READ_ONLY(1);

        int value;

        private static final java.util.Map<Integer, Access> VALUE_MAP;

        Access(int value) {
            this.value = value;
        }

        /**
         * Returns integer value.
         *
         * @return integer value
         */
        public int intValue() {
            return value;
        }

        static {
            Builder<Integer, Access> b = builder();
            for (Access enumItem : Access.values()) {
                b.put(enumItem.value, enumItem);
            }
            VALUE_MAP = b.build();
        }

        /**
         * Returns the value map.
         *
         * @param value value to get value map
         * @return corresponding IsReadOnly item
         * @throws IllegalArgumentException if there is no mapping
         */
        public static Access forValue(int value) {
            Access is = VALUE_MAP.get(value);
            if (is == null) {
                throw new IllegalArgumentException(
                        "No PropertyValueType for value: " + value);
            }
            return is;
        }
    }
}