/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.object.property.instance.property.values;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import com.google.common.collect.Maps;

/**
 * The builder of integer value.
 */
public class IntValueBuilder {

    private static final long MAX_LENGTH =  4294967295L;
    private static final long MIN_LENGTH =  0L;
    private static final String ERROR_LEN_MSG = "Invalid range:" +
            " %s, expected: [[" + MIN_LENGTH + "‥" + MAX_LENGTH + "]].";

    private Map<Class<? extends IntValue>, IntValue> augmentation =
            Maps.newHashMap();
    private IntValueKey key;
    private long order;
    private long value;

    /**
     * Creates a default user role builder.
     */
    public IntValueBuilder() {
    }

    /**
     * Creates a integer value builder from a integer value object.
     *
     * @param base the integer value object
     */
    public IntValueBuilder(IntValue base) {
        if (base.getKey() == null) {
            key = new IntValueKey(base.getOrder(), base.getValue());
            order = base.getOrder();
            value = base.getValue();
        } else {
            key = base.getKey();
            order = key.getOrder();
            value = key.getValue();
        }
    }

    /**
     * Sets the integer value key to be used by the builder.
     *
     * @param key integer value key
     * @return self
     */
    public IntValueBuilder key(IntValueKey key) {
        this.key = key;
        return this;
    }

    /**
     * Sets the order to be used by the builder.
     *
     * @param order order value
     * @return self
     */
    public IntValueBuilder order(long order) {
        checkOrderRange(order);
        this.order = order;
        return this;
    }

    /**
     * Sets the value to be used by the builder.
     *
     * @param value integer value
     * @return self
     */
    public IntValueBuilder value(long value) {
        this.value = value;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public IntValueBuilder addAugmentation(
            Class<? extends IntValue> augmentationType,IntValue augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public IntValueBuilder removeAugmentation(
            Class<? extends IntValue> augmentationType) {
            augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a integer value from this builder instance.
     *
     * @return integer value
     */
    public IntValue build() {
        return new IntValueImpl(this);
    }

    private static void checkOrderRange(long value) {
        if (value < MIN_LENGTH || value > MAX_LENGTH) {
            throw new IllegalArgumentException(
                    String.format(ERROR_LEN_MSG, value));
        }
    }

    /**
     * The implementation of integer value interface.
     */
    private static final class IntValueImpl implements IntValue {

        private final IntValueKey key;
        private final long order;
        private final long value;
        private final Map<Class<? extends IntValue>, IntValue> augmentation;

        private IntValueImpl(IntValueBuilder base) {
            if (base.key == null) {
                key = new IntValueKey(base.order, base.value);
                order = base.order;
                value = base.value;
            } else {
                key = base.key;
                order = key.getOrder();
                value = key.getValue();
            }
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public IntValueKey getKey() {
            return key;
        }

        @Override
        public long getOrder() {
            return order;
        }

        @Override
        public long getValue() {
            return value;
        }

        @Override
        public int hashCode() {
            return Objects.hash(key, order, value, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            IntValueImpl o = (IntValueImpl) obj;
            return Objects.equals(key, o.key) &&
                    Objects.equals(order, o.order) &&
                    Objects.equals(value, o.value) &&
                    Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("key", key)
                    .add("order", order)
                    .add("value", value)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}