/*
 * Copyright 2014-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.action.instance.parameter.values;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;

/**
 * Builder of range value.
 */
public class RangeValueBuilder {

    private Map<Class<? extends RangeValue>, RangeValue> augmentation =
            new HashMap<>();
    private long max;
    private long min;

    /**
     * Creates object by default.
     */
    public RangeValueBuilder() {
    }

    /**
     * Creates a copy from Source Object.
     *
     * @param base Source object
     */
    public RangeValueBuilder(RangeValue base) {
        max = base.getMax();
        min = base.getMin();
    }

    /**
     * Sets maximum value of the instance.
     *
     * @param value maximum value
     * @return self
     */
    public RangeValueBuilder max(long value) {
        max = value;
        return this;
    }

    /**
     * Sets minimum value of the instance.
     *
     * @param value minimum value
     * @return self
     */
    public RangeValueBuilder min(long value) {
        min = value;
        return this;
    }

    /**
     * Adds augmentation from RangeValue.
     * Remove the augmentation, if the range value is null.
     *
     * @param augmentationType class of range value
     * @param augmentation range value
     * @return self
     */
    public RangeValueBuilder addAugmentation(
            Class<? extends RangeValue> augmentationType,
            RangeValue augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation.
     *
     * @param augmentationType class of range value
     * @return self
     */
    public RangeValueBuilder
            removeAugmentation(Class<? extends RangeValue> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds the range value instance.
     *
     * @return implement of range value
     */
    public RangeValue build() {
        return new RangeValueImpl(this);
    }

    /**
     * Implement of RangeValue.
     */
    private static final class RangeValueImpl implements RangeValue {

        private final long max;
        private final long min;
        private final Map<Class<? extends RangeValue>, RangeValue> augmentation;

        private RangeValueImpl(RangeValueBuilder base) {
            max = base.max;
            min = base.min;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public long getMax() {
            return max;
        }

        @Override
        public long getMin() {
            return min;
        }

        @Override
        public int hashCode() {
            return Objects.hash(max, min, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            RangeValueImpl o = (RangeValueImpl) obj;
            return max == o.max &&
                    min == o.min &&
                    Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("max", max)
                    .add("min", min)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}